#ifndef AMGCL_C_DI_H
#define AMGCL_C_DI_H

#ifdef __cplusplus
extern "C" {
#endif
  
#ifndef AMGCLC_INFO
#define AMGCLC_INFO  
  
/** 
    \brief Info structure
*/
  typedef struct {
    int iters;
    double residual;
  }  amgclcInfo;
#endif
  
/** 
    \brief algebraic multigrid preconditioned Krylov subspace iterative solver.
    
*/
  typedef struct{ void *handle; int blocksize;} amgclcDIAMGSolver;
  amgclcDIAMGSolver amgclcDIAMGSolverCreate(int n, int *ia, int *ja, double *a, int blocksize, char *params);
  amgclcInfo amgclcDIAMGSolverApply(amgclcDIAMGSolver solver, double *sol, double *rhs);
  void amgclcDIAMGSolverDestroy(amgclcDIAMGSolver solver);
  
/** 
    \brief Single level relaxation preconditioned Krylov subspace iterative solver.
*/
  typedef struct{ void *handle; int blocksize;} amgclcDIRLXSolver;
  amgclcDIRLXSolver amgclcDIRLXSolverCreate(int n, int *ia, int *ja, double *a, int blocksize, char *params);
  amgclcInfo amgclcDIRLXSolverApply(amgclcDIRLXSolver solver, double *sol, double *rhs);
  void amgclcDIRLXSolverDestroy(amgclcDIRLXSolver solver);
  
/** 
    \brief One algebraic multigrid preconditioning step.
*/
  typedef struct{ void *handle; int blocksize;} amgclcDIAMGPrecon;
  amgclcDIAMGPrecon amgclcDIAMGPreconCreate(int n, int *ia, int *ja, double *a, int blocksize, char *params);
  void amgclcDIAMGPreconApply(amgclcDIAMGPrecon solver, double *sol, double *rhs);
  void amgclcDIAMGPreconDestroy(amgclcDIAMGPrecon solver);
  
/** 
    \brief One single level relaxation  preconditioning step.
*/
  typedef struct{ void *handle; int blocksize;} amgclcDIRLXPrecon;
  amgclcDIRLXPrecon amgclcDIRLXPreconCreate(int n, int *ia, int *ja, double *a, int blocksize, char *params);
  void amgclcDIRLXPreconApply(amgclcDIRLXPrecon solver, double *sol, double *rhs);
  void amgclcDIRLXPreconDestroy(amgclcDIRLXPrecon solver);
  
  
  
#ifdef __cplusplus
} // extern "C"
#endif

#endif
